@@ -12,7 +12,7 @@ use Sys::Syslog qw(:macros);
 use LISM::Storage;
 use Data::Dumper;
 
-our $VERSION = '2.1.0';
+our $VERSION = '2.1.1';
 
 our $lism_master = 'lism_master';
 our $syncrdn = 'cn=sync';
@@ -180,7 +180,7 @@ sub init
         }
 
         if (!defined($self->{_storage})) {$self->{_storage} = {}};
-        eval "\$self->{_storage}{$dname} = new $module(\'$data->{suffix}\', \'$data->{contentry}\')";
+        eval "\$self->{_storage}{$dname} = new $module(\'$data->{suffix}\', \'$data->{contentrystr}\')";
         if ($@) {
             $self->log(level => 'alert', message => "Can't create $module: $@");
             warn $@;
@@ -407,9 +407,9 @@ sub add
     my $conf = $self->{_lism};
     my $rc;
 
+    $entryStr =~ s/\n\s*([^:]+)\n/$1\n/g;
     my ($dn) = ($entryStr =~ /^dn: (.*)$/m);
     $entryStr =~ s/^dn:.*\n//;
-    $entryStr =~ s/\n\s*([^:]+)\n/$1\n/g;
     $dn = lc $dn ;
     $dn =~ s/,\s+/,/g;
 
@@ -571,11 +571,11 @@ sub _checkConfig
 
         # set container entry
         my $entry;
-        if (!($entry = LISM::Storage->containerEntry($conf->{basedn}, $dconf->{container}[0]))) {
+        if (!($entry = LISM::Storage->buildEntryStr($conf->{basedn}, $dconf->{container}[0]))) {
             $self->log(level => 'alert', message => "$dname data container entry is invalid");
             return 1;
         }
-        $self->{data}{$dname}->{contentry} = $entry;
+        $self->{data}{$dname}->{contentrystr} = $entry;
 
         $self->{data}{$dname}->{conf} = $dconf;
     }
@@ -596,7 +596,7 @@ sub _checkConfig
             # normalize dn
             my $master_suffix = $sync->{master}[0]->{containerdn}[0].','.$conf->{basedn};
             $master->{suffix} = lc $master_suffix;
-            ($master->{contentry} = $src_data->{contentry}) =~ s/$src_data->{suffix}$/$master_suffix/mi;
+            ($master->{contentrystr} = $src_data->{contentrystr}) =~ s/$src_data->{suffix}$/$master_suffix/mi;
             $master->{conf} = $src_data->{conf};
         }
 
@@ -612,6 +612,15 @@ sub _checkConfig
             }
 
             my $sdata = $sync->{data}{$dname};
+
+            # synchronization operation
+            if (!defined($sdata->{syncop})) {
+                $sdata->{syncop} = ['add', 'modify', 'delete'];
+            }
+            if (!defined($sdata->{masterop})) {
+                $sdata->{masterop} = ['add', 'modify', 'delete'];
+            }
+
             foreach my $oname (keys %{$sdata->{object}}) {
                 my $sobject = $sdata->{object}{$oname};
 
@@ -622,13 +631,22 @@ sub _checkConfig
                 }
                 $sobject->{syncdn}[0] = lc $sobject->{syncdn}[0];
 
+                # synchronization attributes
                 if (defined($sobject->{syncattr})) {
                     foreach my $attr (@{$sobject->{syncattr}}) {
+                        if (!defined($attr->{name})) {
+                            $self->log(level => 'alert', message => "sync attribute name doesn't exist");
+                            return 1;
+                        }
                         push(@{$sobject->{syncattrs}}, $attr->{name}[0]);
                     }
                 }
                 if (defined($sobject->{masterattr})) {
                     foreach my $attr (@{$sobject->{masterattr}}) {
+                        if (!defined($attr->{name})) {
+                            $self->log(level => 'alert', message => "master attribute name doesn't exist");
+                            return 1;
+                        }
                         push(@{$sobject->{masterattrs}}, $attr->{name}[0]);
                     }
                 }
@@ -984,7 +1002,7 @@ sub _doSync
         }
 
         # operation should be synchronized or not
-        if (!defined($sdata->{syncop}) || !$ops{$func}) {
+        if (!$ops{$func}) {
             next;
         }
 
@@ -1020,18 +1038,31 @@ sub _doSync
                 $dinfo[0] = $dinfo[0].join("\n", ($info[0] =~ /^($attr: .*)$/gmi))."\n";
             }
 
-            for (my $i = 0; $i < @{$sobject->{syncattrs}}; $i++) {
-                my $attr = $sobject->{syncattrs}[$i];
+            my @sync_attrs;
+            if (defined($sobject->{syncattrs})) {
+                @sync_attrs = @{$sobject->{syncattrs}};
+            } else {
+                (@sync_attrs) = ($info[0] =~ /^([^:]+):/gmi);
+            }
+
+            for (my $j = 0; $j < @sync_attrs; $j++) {
+                my $attr = $sync_attrs[$j];
+                my $sattr;
+ 
                 if ($attr =~ /^(objectClass|$rdn_attr)$/i) {
                     next;
                 }
 
+                if (defined($sobject->{syncattr})) {
+                    $sattr = $sobject->{syncattr}[$j];
+                }
+
                 my @values = $info[0] =~ /^$attr: (.*)$/gmi;
                 if (!@values) {
                     next;
                 }
 
-                my @sync_vals = $self->_checkSyncAttrs($master, $data, $sobject->{syncattr}[$i], @values);
+                my @sync_vals = $self->_checkSyncAttrs($master, $data, $sattr, @values);
 
                 if (@sync_vals) {
                     foreach my $value (@sync_vals) {
@@ -1049,21 +1080,30 @@ sub _doSync
                 my $action = shift @tmp;
                 my $attr   = lc(shift @tmp);
                 my @values;
+                my $sattr;
 
                 while (@tmp > 0 && $tmp[0] ne "ADD" && $tmp[0] ne "DELETE" && $tmp[0] ne "REPLACE") {
                     push(@values, shift @tmp);
                 }
 
-                for (my $i = 0; $i < @{$sobject->{syncattrs}}; $i++) {
-                    if ($sobject->{syncattrs}[$i] =~ /^$attr$/i) {
-                        my @sync_vals = $self->_checkSyncAttrs($master, $data, $sobject->{syncattr}[$i], @values);
-                        if (@sync_vals) {
-                            push(@dinfo, $action, $attr, @sync_vals);
-                        } elsif ($action eq "DELETE") {
-                            push(@dinfo, $action, $attr);
+                if (defined($sobject->{syncattrs})) {
+                    for (my $i = 0; $i < @{$sobject->{syncattrs}}; $i++) {
+                        if ($attr =~ /^$sobject->{syncattrs}[$i]$/i) {
+                            $sattr = $sobject->{syncattr}[$i];
+                            last;
                         }
-                        last;
                     }
+
+                    if (!$sattr) {
+                        next;
+                    }
+                }
+
+                my @sync_vals = $self->_checkSyncAttrs($master, $data, $sattr, @values);
+                if (@sync_vals) {
+                    push(@dinfo, $action, $attr, @sync_vals);
+                } elsif ($action eq "DELETE" && !@values) {
+                    push(@dinfo, $action, $attr);
                 }
             }
 
@@ -1171,7 +1211,7 @@ sub _getSyncInfo
                 $ops{modify} = 0;
                 $ops{delete} = 0;
 
-                if (!defined($sobject->{masterdn}) || !defined($sobject->{masterattrs})) {
+                if (!defined($sobject->{masterdn})) {
                     next;
                 }
 
@@ -1186,9 +1226,9 @@ sub _getSyncInfo
                 }
 
                 # get values from data storage
-                my ($rc, @entries) = $self->_do_search($dbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0, @{$sobject->{masterattrs}});
+                my ($rc, @entries) = $self->_do_search($dbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0);
                 if ($rc) {
-                    $self->log(level => 'error', message => "Can't get master values of $dname($rc)");
+                    $self->log(level => 'error', message => "Can't get values of $dname($rc)");
                     return ($rc, ());
                 }
 
@@ -1198,6 +1238,12 @@ sub _getSyncInfo
                     $masterfilter = Net::LDAP::Filter->new($sobject->{masterfilter}[0]);
                 }
 
+                # synchronized attributes
+                my @sync_attrs;
+                if (defined($sobject->{masterattrs})) {
+                    @sync_attrs = @{$sobject->{masterattrs}};
+                }
+
                 # comare data storage's values with master one
                 for (my $i = 0; $i < @entries; $i++) {
                     my ($dn) = ($entries[$i] =~ /^dn: (.*)\n/);
@@ -1219,16 +1265,25 @@ sub _getSyncInfo
                             $nosync_entries{lc($dn)} = 1;
                         }
                     } else {
-                        for (my $j = 0; $j < @{$sobject->{masterattrs}}; $j++) {
-                            my $attr = $sobject->{masterattrs}[$j];
+                        if (!defined($sobject->{masterattrs})) {
+                            @sync_attrs = $self->_unique(($present_list->{$sbase}{$key}->{entryStr} =~ /^([^:]+):/gmi), ($entries[$i] =~ /\n([^:]+):/gi));
+                        }
+
+                        for (my $j = 0; $j < @sync_attrs; $j++) {
+                            my $attr = $sync_attrs[$j];
+                            my $sattr;
                             my @values;
 
+                            if (defined($sobject->{masterattr})) {
+                                $sattr = $sobject->{masterattr}[$j];
+                            }
+
                             @values = $entries[$i] =~ /^$attr: (.*)$/gmi;
-                            my @sync_vals = $self->_checkSyncAttrs($data, $master, $sobject->{masterattr}[$j], @values);
+                            my @sync_vals = $self->_checkSyncAttrs($data, $master, $sattr, @values);
                             my $dvals = join("\n", sort(@sync_vals));
 
                             @values = $present_list->{$sbase}{$key}->{entryStr} =~ /^$attr: (.*)$/gmi;
-                            my ($synced_vals, $left_vals) = $self->_checkSyncedAttrs($master, $data, $sobject->{masterattr}[$j], @values);
+                            my ($synced_vals, $left_vals) = $self->_checkSyncedAttrs($master, $data, $sattr, @values);
                             my $pvals = join("\n", @{$synced_vals});
 
                             # ignore passowrd equality if hash type is differnt
@@ -1278,10 +1333,6 @@ sub _getSyncInfo
                 $ops{modify} = 0;
                 $ops{delete} = 0;
 
-                if (!defined($sobject->{syncattrs})) {
-                    next;
-                }
-
                 foreach my $op (@{$sdata->{syncop}}) {
                     $ops{$op} = 1;
                 }
@@ -1290,9 +1341,9 @@ sub _getSyncInfo
                 my $sbase = $sobject->{syncdn}[0].','.$master->{suffix};
 
                 # get values from data storage
-                my ($rc, @entries) = $self->_do_search($dbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0, @{$sobject->{syncattrs}});
+                my ($rc, @entries) = $self->_do_search($dbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0);
                 if ($rc) {
-                    $self->log(level => 'error', message => "Can't get master values of $dname($rc)");
+                    $self->log(level => 'error', message => "Can't get values of $dname($rc)");
                     return ($rc, ());
                 }
 
@@ -1302,6 +1353,12 @@ sub _getSyncInfo
                     $syncfilter = Net::LDAP::Filter->new($sobject->{syncfilter}[0]);
                 }
 
+                # synchronized attributes
+                my @sync_attrs;
+                if (defined($sobject->{syncattrs})) {
+                    @sync_attrs = @{$sobject->{syncattrs}};
+                }
+
                 # comare data storage's values with master one
                 for (my $i = 0; $i < @entries; $i++) {
                     my ($dn) = ($entries[$i] =~ /^dn: (.*)\n/);
@@ -1316,18 +1373,31 @@ sub _getSyncInfo
                         if ($ops{delete}) {
                             $nosync_entries{lc($dn)} = 1;
                         }
-                    } elsif ($present_list->{$sbase}{$key} && defined($syncfilter) ||
-                        LISM::Storage->parseFilter($syncfilter, $present_list->{$sbase}{$key}->{entryStr})) {
-                        for (my $j = 0; $j < @{$sobject->{syncattrs}}; $j++) {
-                            my $attr = $sobject->{syncattrs}[$j];
+                    } elsif ($present_list->{$sbase}{$key} && !defined($syncfilter) ||
+                            LISM::Storage->parseFilter($syncfilter, $present_list->{$sbase}{$key}->{entryStr})) {
+                        if (!defined($sobject->{syncattrs})) {
+                            @sync_attrs = $self->_unique(($present_list->{$sbase}{$key}->{entryStr} =~ /^([^:]+):/gmi), ($entries[$i] =~ /\n([^:]+):/gi));
+                        }
+
+                        for (my $j = 0; $j < @sync_attrs; $j++) {
+                            my $attr = $sync_attrs[$j];
+                            my $sattr;
                             my @values;
 
+                            if (defined($sobject->{syncattr})) {
+                                $sattr = $sobject->{syncattr}[$j];
+                            }
+
+                            if (defined($sattr->{check}) && $sattr->{check}[0] eq 'off') {
+                                next;
+                            }
+
                             @values = $present_list->{$sbase}{$key}->{entryStr} =~ /^$attr: (.*)$/gmi;
-                            my @sync_vals = $self->_checkSyncAttrs($master, $data, $sobject->{syncattr}[$j], @values);
+                            my @sync_vals = $self->_checkSyncAttrs($master, $data, $sattr, @values);
                             my $pvals = join("\n", @sync_vals);
 
                             @values = $entries[$i] =~ /^$attr: (.*)$/gmi;
-                            my ($synced_vals, $left_vals) = $self->_checkSyncedAttrs($data, $master, $sobject->{syncattr}[$j], @values);
+                            my ($synced_vals, $left_vals) = $self->_checkSyncedAttrs($data, $master, $sattr, @values);
                             my $dvals = join("\n", sort(@{$synced_vals}));
 
                             # ignore passowrd equality if hash type is differnt
@@ -1343,7 +1413,7 @@ sub _getSyncInfo
                         }
                     }
 
-                    if ($ops{add}) {
+                    if ($ops{add} && defined($present_list->{$sbase}{$key})) {
                         $present_list->{$sbase}{$key}->{sync_present}{$dname} = 1;
                     }
                 }
@@ -1417,7 +1487,7 @@ sub _setSyncInfo
                 $ops{modify} = 0;
                 $ops{delete} = 0;
 
-                if (!defined($sobject->{masterdn}) || !defined($sobject->{masterattrs})) {
+                if (!defined($sobject->{masterdn})) {
                     next;
                 }
 
@@ -1431,10 +1501,10 @@ sub _setSyncInfo
                     }
                 }
 
-                # get master values from data storage
-                my ($rc, @entries) = $self->search($dbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0, ());
+                # get values from data storage
+                my ($rc, @entries) = $self->search($dbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0);
                 if ($rc) {
-                    $self->log(level => 'error', message => "Can't get master values of $dname($rc)");
+                    $self->log(level => 'error', message => "Can't get values of $dname($rc)");
                     return $rc;
                 }
 
@@ -1444,6 +1514,12 @@ sub _setSyncInfo
                     $masterfilter = Net::LDAP::Filter->new($sobject->{masterfilter}[0]);
                 }
 
+                # synchronized attributes
+                my @sync_attrs;
+                if (defined($sobject->{masterattrs})) {
+                    @sync_attrs = @{$sobject->{masterattrs}};
+                }
+
                 # comare data storage's values with master one
                 for (my $i = 0; $i < @entries; $i++) {
                     # check need for synchronization
@@ -1468,18 +1544,28 @@ sub _setSyncInfo
                             $entryStr = $entryStr.join("\n", ($entries[$i] =~ /^($attr: .*)$/gmi))."\n";
                         }
 
-                        for (my $j = 0; $j < @{$sobject->{masterattrs}}; $j++) {
-                            my $attr = $sobject->{masterattrs}[$j];
+                        if (!defined($sobject->{masterattrs})) {
+                            (@sync_attrs) = ($entries[$i] =~ /^([^:]+):/gmi);
+                        }
+
+                        for (my $j = 0; $j < @sync_attrs; $j++) {
+                            my $attr = $sync_attrs[$j];
+                            my $sattr;
+
                             if ($attr =~ /^(objectClass|$rdn_attr)$/i) {
                                 next;
                             }
 
+                            if (defined($sobject->{masterattr})) {
+                                $sattr = $sobject->{masterattr}[$j];
+                            }
+
                             my @values = $entries[$i] =~ /^$attr: (.*)$/gmi;
                             if (!@values) {
                                 next;
                             }
 
-                            my @sync_vals = $self->_checkSyncAttrs($data, $master, $sobject->{masterattr}[$j], @values);
+                            my @sync_vals = $self->_checkSyncAttrs($data, $master, $sattr, @values);
 
                             if (@sync_vals) {
                                 foreach my $value (@sync_vals) {
@@ -1489,6 +1575,7 @@ sub _setSyncInfo
                         }
 
                         if ($ops{add}) {
+                            $dn = encode('utf8', $dn);
                             $entryStr = encode('utf8', $entryStr);
                             $rc = $self->_doUpdate('add', $dname, 1, $dn, $entryStr);
                             $present_list->{$sbase}{$key}->{entryStr} = $entryStr;
@@ -1498,16 +1585,25 @@ sub _setSyncInfo
                         my @modlist;
                         my $pocs;
 
-                        for (my $j = 0; $j < @{$sobject->{masterattrs}}; $j++) {
+                        if (!defined($sobject->{masterattrs})) {
+                            @sync_attrs = $self->_unique(($present_list->{$sbase}{$key}->{entryStr} =~ /^([^:]+):/gmi), ($entries[$i] =~ /\n([^:]+):/gi));
+                        }
+
+                        for (my $j = 0; $j < @sync_attrs; $j++) {
                             my $attr = $sobject->{masterattrs}[$j];
+                            my $sattr;
                             my @values;
 
+                            if (defined($sobject->{masterattr})) {
+                                $sattr = $sobject->{masterattr}[$j];
+                            }
+
                             @values = $entries[$i] =~ /^$attr: (.*)$/gmi;
-                            my @sync_vals = $self->_checkSyncAttrs($data, $master, $sobject->{masterattr}[$j], @values);
+                            my @sync_vals = $self->_checkSyncAttrs($data, $master, $sattr, @values);
                             my $dvals = join("\n", sort(@sync_vals));
 
                             @values = $present_list->{$sbase}{$key}->{entryStr} =~ /^$attr: (.*)$/gmi;
-                            my ($synced_vals, $left_vals) = $self->_checkSyncedAttrs($master, $data, $sobject->{masterattr}[$j], @values);
+                            my ($synced_vals, $left_vals) = $self->_checkSyncedAttrs($master, $data, $sattr, @values);
                             my $pvals = join("\n", @{$synced_vals});
 
                             # ignore passowrd equality if hash type is differnt
@@ -1549,6 +1645,7 @@ sub _setSyncInfo
 
                         if (@modlist) {
                             if ($ops{modify}) {
+                                $dn = encode('utf8', $dn);
                                 $rc = $self->_doUpdate('modify', $dname, 1, $dn, @modlist);
                             }
                         }
@@ -1571,7 +1668,7 @@ sub _setSyncInfo
             foreach my $key (keys %{$present_list->{$sbase}}) {
                 if (!$present_list->{$sbase}{$key}->{present}) {
                     my $nosync_entry = "$key,$sbase";
-
+                    $nosync_entry = encode('utf8', $nosync_entry);
                     $rc = $self->_doUpdate('delete', undef, 1, $nosync_entry);
 
                     if ($rc) {
@@ -1603,10 +1700,6 @@ sub _setSyncInfo
                 $ops{modify} = 0;
                 $ops{delete} = 0;
 
-                if (!defined($sobject->{syncattrs})) {
-                    next;
-                }
-
                 foreach my $op (@{$sdata->{syncop}}) {
                     $ops{$op} = 1;
                     if ($op eq 'delete') {
@@ -1617,10 +1710,10 @@ sub _setSyncInfo
                 my $dbase = $sobject->{syncdn}[0].','.$data->{suffix};
                 my $sbase = $sobject->{syncdn}[0].','.$master->{suffix};
 
-                # get master values from data storage
-                my ($rc, @entries) = $self->search($dbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0, ());
+                # get values from data storage
+                my ($rc, @entries) = $self->search($dbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0);
                 if ($rc) {
-                    $self->log(level => 'error', message => "Can't get master values of $dname($rc)");
+                    $self->log(level => 'error', message => "Can't get values of $dname($rc)");
                     return $rc;
                 }
 
@@ -1630,9 +1723,14 @@ sub _setSyncInfo
                     $syncfilter = Net::LDAP::Filter->new($sobject->{syncfilter}[0]);
                 }
 
+                # synchronized attributes
+                my @sync_attrs;
+                if (defined($sobject->{syncattrs})) {
+                    @sync_attrs = @{$sobject->{syncattrs}};
+                }
+
                 # comare data storage's values with master one
                 for (my $i = 0; $i < @entries; $i++) {
-                    $entries[$i] = encode('utf8', $entries[$i]);
                     my ($dn) = ($entries[$i] =~ /^dn: (.*)\n/);
                     my ($key) = ($dn =~ /^(.*),$dbase$/);
                     if (!$key) {
@@ -1643,6 +1741,7 @@ sub _setSyncInfo
                     if (!defined($present_list->{$sbase}{$key})) {
                         if ($ops{delete}) {
                             # delete entry which don't exist in master storage
+                            $dn = encode('utf8', $dn);
                             $rc = $self->_doUpdate('delete', undef, 1, $dn);
                         }
                     } elsif (!defined($syncfilter) ||
@@ -1651,16 +1750,29 @@ sub _setSyncInfo
                         my @modlist;
                         my $pocs;
 
-                        for (my $j = 0; $j < @{$sobject->{syncattrs}}; $j++) {
-                            my $attr = $sobject->{syncattrs}[$j];
+                        if (!defined($sobject->{syncattrs})) {
+                            @sync_attrs = $self->_unique(($present_list->{$sbase}{$key}->{entryStr} =~ /^([^:]+):/gmi), ($entries[$i] =~ /\n([^:]+):/gi));
+                        }
+
+                        for (my $j = 0; $j < @sync_attrs; $j++) {
+                            my $attr = $sync_attrs[$j];
+                            my $sattr;
                             my @values;
 
+                            if (defined($sobject->{syncattr})) {
+                                $sattr = $sobject->{syncattr}[$j];
+                            }
+
+                            if (defined($sattr->{check}) && $sattr->{check}[0] eq 'off') {
+                                next;
+                            }
+
                             @values = $present_list->{$sbase}{$key}->{entryStr} =~ /^$attr: (.*)$/gmi;
-                            my @sync_vals = $self->_checkSyncAttrs($master, $data, $sobject->{syncattr}[$j], @values);
+                            my @sync_vals = $self->_checkSyncAttrs($master, $data, $sattr, @values);
                             my $pvals = join("\n", @sync_vals);
 
                             @values = $entries[$i] =~ /^$attr: (.*)$/gmi;
-                            my ($synced_vals, $left_vals) = $self->_checkSyncedAttrs($data, $master, $sobject->{syncattr}[$j], @values);
+                            my ($synced_vals, $left_vals) = $self->_checkSyncedAttrs($data, $master, $sattr, @values);
                             my $dvals = join("\n", sort(@{$synced_vals}));
 
                             # ignore passowrd equality if hash type is differnt
@@ -1687,6 +1799,7 @@ sub _setSyncInfo
 
                         if (@modlist) {
                             if ($ops{modify}) {
+                                $dn = encode('utf8', $dn);
                                 $rc = $self->_doUpdate('modify', undef, 1, $dn, @modlist);
                             }
                         }
@@ -1697,7 +1810,7 @@ sub _setSyncInfo
                         return $rc;
                     }
 
-                    if ($ops{add}) {
+                    if ($ops{add} && defined($present_list->{$sbase}{$key})) {
                         $present_list->{$sbase}{$key}->{sync_present}{$dname} = 1;
                     }
                 }
@@ -1717,18 +1830,28 @@ sub _setSyncInfo
                                 $entryStr = $entryStr.join("\n", ($present_list->{$sbase}{$key}->{entryStr} =~ /^($attr: .*)$/gmi))."\n";
                             }
 
-                            for (my $j = 0; $j < @{$sobject->{syncattrs}}; $j++) {
-                                my $attr = $sobject->{syncattrs}[$j];
+                            if (!defined($sobject->{syncattrs})) {
+                                @sync_attrs = ($present_list->{$sbase}{$key}->{entryStr} =~ /^([^:]+):/gmi);
+                            }
+
+                            for (my $j = 0; $j < @sync_attrs; $j++) {
+                                my $attr = $sync_attrs[$j];
+                                my $sattr;
+
                                 if ($attr =~ /^(objectClass|$rdn_attr)$/i) {
                                     next;
                                 }
 
+                                if (defined($sobject->{syncattr})) {
+                                   $sattr = $sobject->{syncattr}[$j];
+                                }
+
                                 my @values = $present_list->{$sbase}{$key}->{entryStr} =~ /^$attr: (.*)$/gmi;
                                 if (!@values) {
                                     next;
                                 }
 
-                                my @sync_vals = $self->_checkSyncAttrs($master, $data, $sobject->{syncattr}[$j], @values);
+                                my @sync_vals = $self->_checkSyncAttrs($master, $data, $sattr, @values);
                                 if (@sync_vals) {
                                     foreach my $value (@sync_vals) {
                                         $entryStr = "$entryStr$attr: $value\n";
@@ -1736,6 +1859,7 @@ sub _setSyncInfo
                                 }
                             }
 
+                            $nosync_entry = encode('utf8', $nosync_entry);
                             $entryStr = encode('utf8', $entryStr);
                             $rc = $self->_doUpdate('add', undef, 1, $nosync_entry, $entryStr);
                             if ($rc) {
@@ -1775,6 +1899,7 @@ sub _getPresentList
                     if (!$key) {
                         next;
                     }
+                    $key = lc($key);
 
                     my $entryStr;
                     ($entryStr = $entries[$i]) =~ s/^dn:.*\n//;
@@ -1794,10 +1919,9 @@ sub _checkSyncAttrs
     my ($src_data, $dst_data, $sattr, @values) = @_;
     my @sync_vals = ();
 
-    my $attr = $sattr->{name}[0];
     foreach my $value (@values) {
         # check attribute synchronization rule
-        if (defined($sattr->{rule})) {
+        if ($sattr && defined($sattr->{rule})) {
             my $doSyncAttr = 1;
             foreach my $rule (@{$sattr->{rule}}) {
                 if ($value !~ /$rule/i) {
@@ -1832,12 +1956,11 @@ sub _checkSyncedAttrs
     my @synced_vals = ();
     my @left_vals = ();
 
-    my $attr = $sattr->{name}[0];
     foreach my $value (@values) {
         my $doSyncAttr = 1;
 
         # check attribute synchronization rule
-        if (defined($sattr->{rule})) {
+        if ($sattr && defined($sattr->{rule})) {
             my $tmpval = $value;
 
             # replace suffix of dn values
@@ -1876,6 +1999,16 @@ sub _cmpPwdHash
     }
 }
 
+sub _unique
+{
+    my $self = shift;
+    my @array = @_;
+
+    my %hash = map {$_ => 1} @array;
+
+    return keys %hash;
+}
+
 sub _getDataName
 {
     my $self = shift;
