@@ -146,16 +146,26 @@ sub post_search
 
                 my (@line) = split(/\n/, ${$entriesp}[$i]);
                 my ($dn) = ($line[0] =~ /^dn: (.*)$/);
-                for (my $j = 0; $j < @line; $j++) {
+
+                $line[0] = $self->_rewriteParse($rule->{match}, $substitution, $line[0], \%rwcache);
+                if (!$line[0]) {
+                    $self->log(level => 'error', message => "search result rewrite \"$dn\" failed");
+                    next;
+                }
+
+                my @replaced;
+                for (my $j = 1; $j < @line; $j++) {
                     $line[$j] = $self->_rewriteParse($rule->{match}, $substitution, $line[$j], \%rwcache);
                     if (!defined($line[$j])) {
                         $self->log(level => 'error', message => "search result rewrite \"$dn\" failed");
                         $rc = -1;
                         last;
+                    } elsif ($line[$j]) {
+                        push(@replaced, $line[$j]);
                     }
                 }
                 if (!$rc) {
-                    ${$entriesp}[$i] = join("\n", @line)."\n";
+                    ${$entriesp}[$i] = "$line[0]\n".join("\n", $self->_unique(@replaced))."\n";
                 }
             }
         }
@@ -192,21 +202,30 @@ sub pre_modify
             }
             ${$dnp} = $str;
 
-            for (my $i = 0; $i < @{$listp};) {
-                # skip action
-                $i++;
-                my $attr = ${$listp}[$i++];
+            my @mod_list;
+            while (@{$listp} > 0) {
+                my $action = shift @{$listp};
+                my $attr = shift @{$listp};
+                my @replaced;
 
-                while ($i < @{$listp} && !(${$listp}[$i] =~ /ADD|DELETE|REPLACE/)) {
-                    $str = $self->_rewriteParse($rule->{match}, $substitution, "$attr: ${$listp}[$i]", \%rwcache);
-                    if (!$str) {
+                while (@{$listp} > 0 && ${$listp}[$0] !~ /ADD|DELETE|REPLACE/) {
+                    $str = $self->_rewriteParse($rule->{match}, $substitution, "$attr: ".shift @{$listp}, \%rwcache);
+                    if (!defined($str)) {
                         $self->log(level => 'error', message => "modify rewrite \"${$dnp}\" failed");
                         return -1;
+                    } elsif ($str) {
+                        foreach my $line (split(/\n/, $str)) {
+                            my $value;
+                            ($attr, $value) = split(/: */, $line);
+                            push(@replaced, $value);
+                        }
                     }
-                    ($attr, ${$listp}[$i]) = split(/: */, $str);
-                    $i++;
+                }
+                if (@replaced) {
+                    push(@mod_list, ($action, $attr, @replaced));
                 }
             }
+            @{$listp} = @mod_list;
         }
     }
 
@@ -244,13 +263,14 @@ sub pre_add
 
             for (my $i = 0; $i < @line; $i++) {
                 $line[$i] = $self->_rewriteParse($rule->{match}, $substitution, $line[$i], \%rwcache);
-                if (!$line[$i]) {
+                if (!defined($line[$i])) {
                     $self->log(level => 'error', message => "add rewrite \"${$dnp}\" failed");
                     return -1;
+                } elsif ($line[$i]) {
+                    ${$entryStrp}[0] = join("\n", @line)."\n";
                 }
             }
-            ${$entryStrp}[0] = join("\n", @line)."\n";
-         }
+        }
     }
 
     return 0;
@@ -317,6 +337,7 @@ sub pre_delete
                 $self->log(level => 'error', message => "delete rewrite \"${$dnp}\" failed");
                 return -1;
             }
+            ${$dnp} = $str;
         }
     }
 
@@ -380,31 +401,47 @@ sub _rewriteParse
 
     if ($str =~ /$match/i) {
         # do functions
+        my @subs = ($substitution);
         my @rwmaps = ($substitution =~ /%{([^}]*)}/g);
         foreach my $rwmap (@rwmaps) {
-            my $value;
+            my @values;
             my $key = lc($rwmap);
             if (defined(${$rwcache}{$key})) {
-                $value = ${$rwcache}{$key};
+                @values = @{${$rwcache}{$key}};
             } else {
                 my ($map_name, $map_args) = ($rwmap =~ /^([^(]*)\((.*)\)$/);
-                $value = $self->_rewriteMap($map_name, $map_args);
-                if (!defined($value)) {
+                @values = $self->_rewriteMap($map_name, $map_args);
+                if (!defined(@values)) {
                     return undef;
                 }
 
-                ${$rwcache}{$key} = $value;
+                ${$rwcache}{$key} = \@values;
             }
 
-            if (!$value) {
+            if (!$values[0]) {
                 return $str;
             }
 
             $rwmap =~ s/([()])/\\$1/g;
-            $substitution =~ s/%{$rwmap}/$value/;
+            my @tmpsubs;
+            foreach my $sub (@subs) {
+                foreach my $value (@values) {
+                    my $tmpsub = $sub;
+                    $tmpsub =~ s/%{$rwmap}/$value/;
+                    push(@tmpsubs, $tmpsub);
+                }
+            }
+            undef(@subs);
+            @subs = @tmpsubs;
         }
 
-        $str =~ s/$match/$substitution/gi;
+        my @strs;
+        foreach my $sub (@subs) {
+            my $tmpstr = $str;
+            $tmpstr =~ s/$match/$sub/gi;
+            push(@strs, $tmpstr);
+        }
+        $str = join("\n", @strs);
     }
 
     return $str;
@@ -415,14 +452,14 @@ sub _rewriteMap
     my $self = shift;
     my ($map_name, $map_args) = @_;
     my $conf = $self->{_config};
-    my $value = '';
+    my @values = ();
 
     if (defined($conf->{rewritemap}{$map_name})) {
         my $method = '_'.$conf->{rewritemap}{$map_name}->{type}.'Map';
-        $value = $self->$method($map_name, $map_args);
+        @values = $self->$method($map_name, $map_args);
     }
 
-    return $value;
+    return @values;
 }
 
 sub _ldapMap
@@ -430,25 +467,34 @@ sub _ldapMap
     my $self = shift;
     my ($map_name, $map_args) = @_;
     my $ldapmap = $self->{ldapmap}{$map_name};
+    my $base = $ldapmap->{base};
+    my $scope = 'sub';
+    my $filter = $map_args;
     my $msg;
     my $rc;
-    my $value = '';
+    my @values = ('');
 
     if (!$ldapmap->{ldap}) {
         $ldapmap->{ldap} = Net::LDAP->new($ldapmap->{uri});
         $msg = $ldapmap->{ldap}->bind($ldapmap->{binddn}, password => $ldapmap->{bindpw});
         $rc = $msg->code;
         if ($rc) {
-            $self->log(level => 'level', message => "rewriteMap $map_name bind by $ldapmap->{binddn} failed($rc)");
+            $self->log(level => 'error', message => "rewriteMap $map_name bind by $ldapmap->{binddn} failed($rc)");
             return undef;
         }
     }
 
-    $msg = $ldapmap->{ldap}->search(base => $ldapmap->{base}, filter => $map_args, attrs => [$ldapmap->{attr}]);
+    if ($map_args =~ /^\(?dn=/) {
+        ($base) = ($map_args =~ /^\(?dn=(.*)\)?$/);
+        $scope = 'base';
+        $filter = 'objectClass=*';
+    }
+
+    $msg = $ldapmap->{ldap}->search(base => $base, scope => $scope, filter => $filter, attrs => [$ldapmap->{attr}]);
 
     $rc = $msg->code;
     if ($rc) {
-        $self->log(level => 'level', message => "rewriteMap $map_name search by $map_args failed($rc)");
+        $self->log(level => 'error', message => "rewriteMap $map_name search by $map_args failed($rc)");
         if ($rc == LDAP_SERVER_DOWN) {
             $ldapmap->{ldap}->unbind();
             undef($ldapmap->{ldap});
@@ -459,28 +505,33 @@ sub _ldapMap
 
     if ($msg->count) {
         my $entry = $msg->entry(0);
-        $value = $entry->get_value($ldapmap->{attr});
-        if (!defined($value)) {
-            $value = '';
+        if ($ldapmap->{attr} eq 'dn') {
+            $values[0] = $entry->dn;
+        } else {
+            @values = $entry->get_value($ldapmap->{attr});
+        }
+
+        if (!defined($values[0])) {
+            @values = ('');
         }
     }
 
-    return $value;    
+    return @values;
 }
 
 sub _functionMap
 {
     my $self = shift;
     my ($map_name, $map_args) = @_;
-    my $value;
+    my @values;
 
-    eval "\$value = $map_name($map_args)";
+    eval "\@values = $map_name($map_args)";
     if ($@) {
         $self->log(level => 'error', message => "rewriteMap $map_name failed: $@");
         return undef;
     }
 
-    return $value;
+    return @values;
 }
 
 =head1 SEE ALSO
