@@ -45,12 +45,12 @@ sub commit
     my $self = shift;
     my $conf = $self->{_config};
 
-    foreach my $obj (keys %{$conf->{object}}) {
-        if (!defined($conf->{object}{$obj}->{file})) {
+    foreach my $oname (keys %{$conf->{object}}) {
+        if (!defined($conf->{object}{$oname}->{file})) {
             next;
         }
 
-        my $file = $conf->{object}{$obj}->{file}[0];
+        my $file = $conf->{object}{$oname}->{file}[0];
         if (!open(LOCK, "> $file.lock")) {
             $self->log(level => 'alert', message => "Can't open $file.lock");
             return -1;
@@ -78,8 +78,8 @@ sub rollback
     my $self = shift;
     my $conf = $self->{_config};
 
-    foreach my $obj (keys %{$conf->{object}}) {
-        my $file = $conf->{object}{$obj}->{file}[0];
+    foreach my $oname (keys %{$conf->{object}}) {
+        my $file = $conf->{object}{$oname}->{file}[0];
         if (!open(LOCK, "> $file.lock")) {
             $self->log(level => 'alert', message => "Can't open $file.lock");
             return -1;
@@ -128,6 +128,7 @@ sub _objSearch
     my $self = shift;
     my ($obj, $pkey, $suffix, $sizeLim, $filter) = @_;
     my $conf = $self->{_config};
+    my $oconf = $obj->{conf};
     my $dlm = $conf->{delim}[0];
     my $valdlm = $conf->{valdelim}[0];
     my @match_entries = ();
@@ -137,15 +138,15 @@ sub _objSearch
     DO: {
         my $lock;
         my $file;
-        if (!open($lock, "> $obj->{file}[0].lock")) {
-            $self->log(level => 'alert', message => "Can't open $obj->{file}[0].lock");
+        if (!open($lock, "> $oconf->{file}[0].lock")) {
+            $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
             $rc = LDAP_OPERATIONS_ERROR;
             last DO;
         }
         flock($lock, 1);
 
-        if (!open($file, "< $obj->{file}[0]")) {
-            $self->log(level => 'alert', message => "Can't open $obj->{file}[0]");
+        if (!open($file, "< $oconf->{file}[0]")) {
+            $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
             close($lock);
             $rc = LDAP_OPERATIONS_ERROR;
             last DO;
@@ -155,7 +156,7 @@ sub _objSearch
             chop;
             my @data = split(/$dlm/);
             my $entry;
-            my $rdn_val = $data[$obj->{attr}{$obj->{rdn}[0]}->{column}[0]];
+            my $rdn_val = $data[$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]];
 
             # check the number of returned entries
             if ($sizeLim >= 0 && @match_entries == $sizeLim) {
@@ -164,8 +165,8 @@ sub _objSearch
             }
 
             # entries below suffix
-            if ($pkey && defined($obj->{container}) && !defined($obj->{container}[0]->{rdn})) {
-                my $cur_pkey = $data[$obj->{container}[0]->{idcolumn}[0]];
+            if ($pkey && defined($oconf->{container}) && !defined($oconf->{container}[0]->{rdn})) {
+                my $cur_pkey = $data[$oconf->{container}[0]->{idcolumn}[0]];
 
                 if (!($pkey =~ /^$cur_pkey$/i)) {
                     next;
@@ -173,18 +174,18 @@ sub _objSearch
             }
 
             # get all values of the entry
-            $entry = "dn: $obj->{rdn}[0]=$rdn_val,$suffix\n";
-            foreach my $oc (@{$obj->{oc}}) {
+            $entry = "dn: $oconf->{rdn}[0]=$rdn_val,$suffix\n";
+            foreach my $oc (@{$oconf->{oc}}) {
                 $entry = $entry."objectclass: $oc\n";
             }
-            foreach my $attr (keys %{$obj->{attr}}) {
-                if (defined($obj->{attr}{$attr}->{column})) {
-                    foreach my $val (split(/$valdlm/, $data[$obj->{attr}{$attr}->{column}[0]])) {
+            foreach my $attr (keys %{$oconf->{attr}}) {
+                if (defined($oconf->{attr}{$attr}->{column})) {
+                    foreach my $val (split(/$valdlm/, $data[$oconf->{attr}{$attr}->{column}[0]])) {
 
                         $entry = $entry."$attr: $val\n";
                     }
                 } else {
-                    my $values = $self->_getAttrValues($obj, $attr, split(/$valdlm/, $data[$obj->{attr}{$attr}->{idcolumn}[0]]));
+                    my $values = $self->_getAttrValues($oconf, $attr, split(/$valdlm/, $data[$oconf->{attr}{$attr}->{idcolumn}[0]]));
                     if (!defined($values)) {
                         $rc = LDAP_OPERATIONS_ERROR;
                         last;
@@ -199,7 +200,7 @@ sub _objSearch
             # parse filter
             if ($self->parseFilter($filter, encode('utf8', $entry))) {
                 push(@match_entries, $self->_pwdFormat($entry));
-                push(@match_keys, $data[$obj->{id}[0]->{column}[0]]);
+                push(@match_keys, $data[$oconf->{id}[0]->{column}[0]]);
             }
         }
         close($file);
@@ -222,6 +223,7 @@ sub _objModify
     my $self = shift;
     my ($obj, $pkey, $dn, @list) = @_;
     my $conf = $self->{_config};
+    my $oconf = $obj->{conf};
     my $dlm = $conf->{delim}[0];
     my $valdlm = $conf->{valdelim}[0];
     my $match = 0;
@@ -235,26 +237,26 @@ sub _objModify
     my $lock;
     my $file;
     my $tmp;
-    if (!open($lock, "> $obj->{file}[0].lock")) {
-        $self->log(level => 'alert', message => "Can't open $obj->{file}[0].lock");
+    if (!open($lock, "> $oconf->{file}[0].lock")) {
+        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
         return LDAP_OPERATIONS_ERROR;
     }
     flock($lock, 2);
 
-    if (!rename($obj->{file}[0], "$obj->{file}[0].tmp")) {
-        $self->log(level => 'alert', message => "Can't rename $obj->{file}[0] to $obj->{file}[0].tmp");
+    if (!rename($oconf->{file}[0], "$oconf->{file}[0].tmp")) {
+        $self->log(level => 'alert', message => "Can't rename $oconf->{file}[0] to $oconf->{file}[0].tmp");
         close($lock);
         return LDAP_OPERATIONS_ERROR;
     }
 
-    if (!open($tmp, "< $obj->{file}[0].tmp")) {
-        $self->log(level => 'alert', message => "Can't open $obj->{file}[0].tmp");
+    if (!open($tmp, "< $oconf->{file}[0].tmp")) {
+        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].tmp");
         close($lock);
         return LDAP_OPERATIONS_ERROR;
     }
 
-    if (!open($file, "> $obj->{file}[0]")) {
-        $self->log(level => 'alert', message => "Can't open $obj->{file}[0]");
+    if (!open($file, "> $oconf->{file}[0]")) {
+        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
         close($tmp);
         close($lock);
         return LDAP_OPERATIONS_ERROR;
@@ -266,14 +268,14 @@ sub _objModify
             my @data = split(/$dlm/);
 
             # check the data corresponds to the dn
-            if (!("$valdlm$data[$obj->{attr}{$obj->{rdn}[0]}->{column}[0]]$valdlm" =~ /$valdlm$rdn_val$valdlm/i)) {
+            if (!("$valdlm$data[$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]]$valdlm" =~ /$valdlm$rdn_val$valdlm/i)) {
                 print $file $_."\n";
                 next;
             }
 
             # entries below suffix
-            if ($pkey && defined($obj->{container}) && !defined($obj->{container}[0]->{rdn})) {
-                my $cur_pkey = $data[$obj->{container}[0]->{idcolumn}[0]];
+            if ($pkey && defined($oconf->{container}) && !defined($oconf->{container}[0]->{rdn})) {
+                my $cur_pkey = $data[$oconf->{container}[0]->{idcolumn}[0]];
 
                 if (!($pkey =~ /^$cur_pkey$/i)) {
                     print $file $_."\n";
@@ -292,13 +294,13 @@ sub _objModify
                     push(@values, shift @list);
                 }
 
-                if (!defined($obj->{attr}{$attr})) {
+                if (!defined($oconf->{attr}{$attr})) {
                     next;
                 }
 
                 # can't modify the attribute for rdn
-                if ($attr eq $obj->{rdn}[0]) {
-                    if ($action ne "REPLACE" || join($valdlm, @values) ne $data[$obj->{attr}{$obj->{rdn}[0]}{column}[0]]) {
+                if ($attr eq $oconf->{rdn}[0]) {
+                    if ($action ne "REPLACE" || join($valdlm, @values) ne $data[$oconf->{attr}{$oconf->{rdn}[0]}{column}[0]]) {
                         $rc =  LDAP_CONSTRAINT_VIOLATION;
                         last;
                     }
@@ -311,14 +313,14 @@ sub _objModify
                     }
                 }
 
-                if (defined($obj->{attr}{$attr}->{column})) {
-                    $coln = $obj->{attr}{$attr}->{column}[0];
+                if (defined($oconf->{attr}{$attr}->{column})) {
+                    $coln = $oconf->{attr}{$attr}->{column}[0];
                 } else {
                     my @keys;
-                    $coln = $obj->{attr}{$attr}->{idcolumn}[0];
+                    $coln = $oconf->{attr}{$attr}->{idcolumn}[0];
 
                     # convert the value to object's id
-                    ($rc, @keys) = $self->_getAttrKeys($obj, $attr, @values);
+                    ($rc, @keys) = $self->_getAttrKeys($oconf, $attr, @values);
                     if ($rc) {
                         $self->log(level => 'error', message => "Can't get id of $attr values in the file");
                         $rc = LDAP_OTHER;
@@ -392,6 +394,7 @@ sub _objAdd
     my $self = shift;
     my ($obj, $pkey, $dn,  $entryStr) = @_;
     my $conf = $self->{_config};
+    my $oconf = $obj->{conf};
     my $dlm = $conf->{delim}[0];
     my $valdlm = $conf->{valdelim}[0];
     my $rc = LDAP_SUCCESS;
@@ -405,27 +408,27 @@ sub _objAdd
     my $lock;
     my $file;
     my $tmp;
-    if (!open($lock, "> $obj->{file}[0].lock")) {
-        $self->log(level => 'alert', message => "Can't open $obj->{file}[0].lock");
+    if (!open($lock, "> $oconf->{file}[0].lock")) {
+        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
         return LDAP_OPERATIONS_ERROR;
     }
     flock($lock, 2);
 
     # check whether the entry already exists
-    if (!rename($obj->{file}[0], "$obj->{file}[0].tmp")) {
-        $self->log(level => 'alert', message => "Can't rename $obj->{file}[0] to $obj->{file}[0].tmp");
+    if (!rename($oconf->{file}[0], "$oconf->{file}[0].tmp")) {
+        $self->log(level => 'alert', message => "Can't rename $oconf->{file}[0] to $oconf->{file}[0].tmp");
         close($lock);
         return LDAP_OPERATIONS_ERROR;
     }
 
-    if (!open($tmp, "< $obj->{file}[0].tmp")) {
-        $self->log(level => 'alert', message => "Can't open $obj->{file}[0].tmp");
+    if (!open($tmp, "< $oconf->{file}[0].tmp")) {
+        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].tmp");
         close($lock);
         return LDAP_OPERATIONS_ERROR;
     }
 
-    if (!open($file, "> $obj->{file}[0]")) {
-        $self->log(level => 'alert', message => "Can't open $obj->{file}[0]");
+    if (!open($file, "> $oconf->{file}[0]")) {
+        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
         close($tmp);
         close($lock);
         return LDAP_OPERATIONS_ERROR;
@@ -438,14 +441,13 @@ sub _objAdd
         my @data = split(/$dlm/);
 
         # check the data correspods to the dn
-        if ("$valdlm$data[$obj->{attr}{$obj->{rdn}[0]}->{column}[0]]$valdlm" !~ /$valdlm$rdn_val$valdlm/i) {
+        if ("$valdlm$data[$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]]$valdlm" !~ /$valdlm$rdn_val$valdlm/i) {
             next;
         }
 
         # entries below suffix
-        if ($pkey && defined($obj->{container}) && !defined($obj->{container}[0]->{rdn})) {
-            my $cur_pkey = $data[$obj->{container}[0]->{idcolumn}[0]];
-
+        if ($pkey && defined($oconf->{container}) && !defined($oconf->{container}[0]->{rdn})) {
+            my $cur_pkey = $data[$oconf->{container}[0]->{idcolumn}[0]];
             if ($pkey !~ /^$cur_pkey$/i) {
                 next;
             }
@@ -460,18 +462,18 @@ sub _objAdd
         my @data;
 
         DO: {
-            foreach my $attr (keys %{$obj->{attr}}) {
+            foreach my $attr (keys %{$oconf->{attr}}) {
                 my $coln;
                 my @values = ($entryStr =~ /^$attr:\s(.*)$/gmi);
 
-                if (defined($obj->{attr}{$attr}->{column})) {
-                    $coln = $obj->{attr}{$attr}->{column}[0];
+                if (defined($oconf->{attr}{$attr}->{column})) {
+                    $coln = $oconf->{attr}{$attr}->{column}[0];
                 } else {
                     my @keys;
-                    $coln = $obj->{attr}{$attr}->{idcolumn}[0];
+                    $coln = $oconf->{attr}{$attr}->{idcolumn}[0];
 
                     # convert the value to object's id
-                    ($rc, @keys) = $self->_getAttrKeys($obj, $attr, @values);
+                    ($rc, @keys) = $self->_getAttrKeys($oconf, $attr, @values);
                     if ($rc) {
                         $self->log(level => 'error', message => "Can't get id of $attr values in the file");
                         $rc = LDAP_OTHER;
@@ -486,12 +488,12 @@ sub _objAdd
             }
 
             # add the link with container
-            if (defined($obj->{container}) && defined($obj->{container}[0]->{idcolumn})) {
-                $data[$obj->{container}[0]->{idcolumn}[0]] = $pkey;
+            if (defined($oconf->{container}) && defined($oconf->{container}[0]->{idcolumn})) {
+                $data[$oconf->{container}[0]->{idcolumn}[0]] = $pkey;
             }
 
             # add storage-specific information
-            foreach my $strginfo (@{$obj->{strginfo}}) {
+            foreach my $strginfo (@{$oconf->{strginfo}}) {
                 my $value = $self->_getStrgInfoValue($strginfo, $dn, $entryStr);
 
                 if (defined($strginfo->{column})) {
@@ -525,6 +527,7 @@ sub _objDelete
     my $self = shift;
     my ($obj, $pkey, $dn) = @_;
     my $conf = $self->{_config};
+    my $oconf = $obj->{conf};
     my $dlm = $conf->{delim}[0];
     my $valdlm = $conf->{valdelim}[0];
     my $rc = LDAP_NO_SUCH_OBJECT;
@@ -537,26 +540,26 @@ sub _objDelete
     my $lock;
     my $file;
     my $tmp;
-    if (!open($lock, "> $obj->{file}[0].lock")) {
-        $self->log(level => 'alert', message => "Can't open $obj->{file}[0].lock");
+    if (!open($lock, "> $oconf->{file}[0].lock")) {
+        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
         return LDAP_OPERATIONS_ERROR;
     }
     flock($lock, 2);
 
-    if (!rename($obj->{file}[0], "$obj->{file}[0].tmp")) {
-        $self->log(level => 'alert', message => "Can't rename $obj->{file}[0] to $obj->{file}[0].tmp");
+    if (!rename($oconf->{file}[0], "$oconf->{file}[0].tmp")) {
+        $self->log(level => 'alert', message => "Can't rename $oconf->{file}[0] to $oconf->{file}[0].tmp");
         close($lock);
         return LDAP_OPERATIONS_ERROR;
     }
 
-    if (!open($tmp, "< $obj->{file}[0].tmp")) {
-        $self->log(level => 'alert', message => "Can't open $obj->{file}[0].tmp");
+    if (!open($tmp, "< $oconf->{file}[0].tmp")) {
+        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].tmp");
         close($lock);
         return LDAP_OPERATIONS_ERROR;
     }
 
-    if (!open($file, "> $obj->{file}[0]")) {
-        $self->log(level => 'alert', message => "Can't open $obj->{file}[0]");
+    if (!open($file, "> $oconf->{file}[0]")) {
+        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
         close($tmp);
         close($lock);
         return LDAP_OPERATIONS_ERROR;
@@ -567,14 +570,14 @@ sub _objDelete
         my @data = split(/$dlm/);
 
         # check the data corresponds to the dn
-        if (!("$valdlm$data[$obj->{attr}{$obj->{rdn}[0]}{column}[0]]$valdlm" =~ /$valdlm$rdn_val$valdlm/i)) {
+        if (!("$valdlm$data[$oconf->{attr}{$oconf->{rdn}[0]}{column}[0]]$valdlm" =~ /$valdlm$rdn_val$valdlm/i)) {
             print $file $_."\n";
             next;
         }
 
         # entries below suffix
-        if ($pkey && defined($obj->{container}) && !defined($obj->{container}[0]->{rdn})) {
-            my $cur_pkey = $data[$obj->{container}[0]->{idcolumn}[0]];
+        if ($pkey && defined($oconf->{container}) && !defined($oconf->{container}[0]->{rdn})) {
+            my $cur_pkey = $data[$oconf->{container}[0]->{idcolumn}[0]];
 
             if (!($pkey =~ /^$cur_pkey$/i)) {
                 print $file $_."\n";
@@ -600,27 +603,28 @@ sub _getParentRdn
     my $self = shift;
     my ($obj, $key) = @_;
     my $conf = $self->{_config};
+    my $oconf = $obj->{conf};
     my $dlm = $conf->{delim}[0];
     my $prdn = undef;
     my $pkey = undef;
 
-    if (defined($obj->{container}[0]->{rdn})) {
-        return $obj->{container}[0]->{rdn}[0];
+    if (defined($oconf->{container}[0]->{rdn})) {
+        return $oconf->{container}[0]->{rdn}[0];
     }
-    if (!defined($obj->{container}[0]->{oname})) {
+    if (!defined($oconf->{container}[0]->{oname})) {
         return undef;
     }
 
     my $lock;
     my $file;
-    if (!open($lock, "> $obj->{file}[0].lock")) {
-        $self->log(level => 'alert', message => "Can't open $obj->{file}[0].lock");
+    if (!open($lock, "> $oconf->{file}[0].lock")) {
+        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
         return LDAP_OPERATIONS_ERROR;
     }
     flock($lock, 1);
 
-    if (!open($file, "< $obj->{file}[0]")) {
-        $self->log(level => 'alert', message => "Can't open $obj->{file}[0]");
+    if (!open($file, "< $oconf->{file}[0]")) {
+        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
         close($lock);
         return LDAP_OPERATIONS_ERROR;
     }
@@ -629,22 +633,21 @@ sub _getParentRdn
         my @data = split(/$dlm/);
 
         # check the data corresponds to the object's id
-        if ($data[$obj->{id}[0]->{column}[0]] =~ /^$key$/i) {
-            $pkey = $data[$obj->{container}[0]->{idcolumn}[0]];
+        if ($data[$oconf->{id}[0]->{column}[0]] =~ /^$key$/i) {
+            $pkey = $data[$oconf->{container}[0]->{idcolumn}[0]];
             last;
         }
     }
 
-    my $pobj = $conf->{object}{$obj->{container}[0]->{oname}[0]};
-
-    if (!open($lock, "> $pobj->{file}[0].lock")) {
-        $self->log(level => 'alert', message => "Can't open $pobj->{file}[0].lock");
+    my $poconf = $self->{object}{$obj->{parent}}->{conf};
+    if (!open($lock, "> $poconf->{file}[0].lock")) {
+        $self->log(level => 'alert', message => "Can't open $poconf->{file}[0].lock");
         return undef;
     }
     flock($lock, 1);
 
-    if (!open($file, "< $pobj->{file}[0]")) {
-        $self->log(level => 'alert', message => "Can't open $pobj->{file}[0]");
+    if (!open($file, "< $poconf->{file}[0]")) {
+        $self->log(level => 'alert', message => "Can't open $poconf->{file}[0]");
         close($lock);
         return undef;
     }
@@ -654,8 +657,8 @@ sub _getParentRdn
         my @data = split(/$dlm/);
 
         # check the data corresponds to the object's id
-        if ($data[$pobj->{id}[0]->{column}[0]] =~ /^$pkey$/i) {
-            $prdn = "$pobj->{rdn}[0]=$data[$pobj->{attr}{$pobj->{rdn}[0]}->{column}[0]]";
+        if ($data[$poconf->{id}[0]->{column}[0]] =~ /^$pkey$/i) {
+            $prdn = "$poconf->{rdn}[0]=$data[$poconf->{attr}{$poconf->{rdn}[0]}->{column}[0]]";
             last;
         }
     }
@@ -668,20 +671,22 @@ sub _getParentRdn
 sub _getAttrValues
 {
     my $self = shift;
-    my ($obj, $attr, @keys) = @_;
+    my ($oconf, $attr, @keys) = @_;
     my $conf = $self->{_config};
     my $dlm = $conf->{delim}[0];
-    my $attrobj = undef;
+    my $aobj = undef;
+    my $aoconf = undef;
     my $attrStr = '';
     my @colnums;
     my $filename;
 
-    if (defined($obj->{attr}{$attr}->{oname})) {
-        $attrobj = $conf->{object}->{$obj->{attr}{$attr}->{oname}[0]};
-        $filename = $attrobj->{file}[0];
-    } elsif (defined($obj->{attr}{$attr}->{file})) {
-        @colnums = ($obj->{attr}{$attr}->{value}[0] =~ /%([0-9]+)/g);
-        $filename = $obj->{attr}{$attr}->{file}[0];
+    if (defined($oconf->{attr}{$attr}->{oname})) {
+        $aobj = $self->{object}{$oconf->{attr}{$attr}->{oname}[0]};
+        $aoconf = $aobj->{conf};
+        $filename = $aoconf->{file}[0];
+    } elsif (defined($oconf->{attr}{$attr}->{file})) {
+        @colnums = ($oconf->{attr}{$attr}->{value}[0] =~ /%([0-9]+)/g);
+        $filename = $oconf->{attr}{$attr}->{file}[0];
     } else {
         return undef;
     }
@@ -706,15 +711,15 @@ sub _getAttrValues
 
         # check the data corresponds to the object's id
         for (my $i = 0; $i < @keys; $i++) {
-            if (defined($attrobj)) {
-                if ($data[$attrobj->{id}[0]->{column}[0]] =~ /^$keys[$i]$/i) {
-                    $attrStr = $attrStr."$attr: $attrobj->{rdn}[0]=$data[$attrobj->{attr}{$attrobj->{rdn}[0]}->{column}[0]],".$self->_getParentDn($attrobj, $data[$attrobj->{id}[0]->{column}[0]])."\n";
+            if (defined($aoconf)) {
+                if ($data[$aoconf->{id}[0]->{column}[0]] =~ /^$keys[$i]$/i) {
+                    $attrStr = $attrStr."$attr: $aoconf->{rdn}[0]=$data[$aoconf->{attr}{$aoconf->{rdn}[0]}->{column}[0]],".$self->_getParentDn($aobj, $data[$aoconf->{id}[0]->{column}[0]])."\n";
                     splice(@keys, $i, 1);
                     last;
                 }
             } else {
-                if ($data[$obj->{attr}{$attr}->{id}[0]->{column}[0]] =~ /^$keys[$i]/i) {
-                    my $value = $obj->{attr}{$attr}->{value}[0];
+                if ($data[$oconf->{attr}{$attr}->{id}[0]->{column}[0]] =~ /^$keys[$i]/i) {
+                    my $value = $oconf->{attr}{$attr}->{value}[0];
                     foreach my $coln (@colnums) {
                         $value =~ s/%$coln/$data[$coln]/g;
                     }
@@ -744,48 +749,48 @@ sub _getAttrValues
 sub _getAttrKeys
 {
     my $self = shift;
-    my ($obj, $attr, @values) = @_;
+    my ($oconf, $attr, @values) = @_;
     my $conf = $self->{_config};
     my $dlm = $conf->{delim}[0];
     my @attrkeys = ();
     my $rc = 0;
 
-    if (defined($obj->{attr}{$attr}->{oname})) {
+    if (defined($oconf->{attr}{$attr}->{oname})) {
         for (my $i = 0; $i < @values && $values[$i]; $i++) {
-            my $attrobj;
+            my $aobj;
             my $attrkey;
             my $attrpkey;
 
-            ($rc, $attrobj, $attrpkey) = $self->_getObject($values[$i]);
+            ($rc, $aobj, $attrpkey) = $self->_getObject($values[$i]);
             if ($rc) {
                 return (-1, ());
             }
 
-            ($rc, $attrkey) =$self->_baseSearch($attrobj, $attrpkey, $values[$i], 0, 0, 1, 0, undef, 0, ('dn'));
+            ($rc, $attrkey) =$self->_baseSearch($aobj, $attrpkey, $values[$i], 0, 0, 1, 0, undef, 0, ('dn'));
             if ($rc || !$attrkey) {
                 return (-1, ());
             }
 
             push(@attrkeys, $attrkey);
         }
-    } elsif (defined($obj->{attr}{$attr}->{file})) {
+    } elsif (defined($oconf->{attr}{$attr}->{file})) {
         my $lock;
         my $file;
 
-        if (!open($lock, "> $obj->{attr}{$attr}->{file}[0].lock")) {
-            $self->log(level => 'alert', message => "Can't open $obj->{attr}{$attr}->{file}[0].lock");
+        if (!open($lock, "> $oconf->{attr}{$attr}->{file}[0].lock")) {
+            $self->log(level => 'alert', message => "Can't open $oconf->{attr}{$attr}->{file}[0].lock");
             return (-1, ());
         }
         flock($lock, 1);
 
-        if (!open($file, "< $obj->{attr}{$attr}->{file}[0]")) {
-            $self->log(level => 'alert', message => "Can't open $obj->{attr}{$attr}->{file}[0]");
+        if (!open($file, "< $oconf->{attr}{$attr}->{file}[0]")) {
+            $self->log(level => 'alert', message => "Can't open $oconf->{attr}{$attr}->{file}[0]");
             close($lock);
             return (-1, ());
         }
 
-        my @colnums = ($obj->{attr}{$attr}->{value}[0] =~ /%([0-9]+)/g);
-        (my $replace = $obj->{attr}{$attr}->{value}[0]) =~ s/([*+\/\.^$()\[\]])/\\$1/g;
+        my @colnums = ($oconf->{attr}{$attr}->{value}[0] =~ /%([0-9]+)/g);
+        (my $replace = $oconf->{attr}{$attr}->{value}[0]) =~ s/([*+\/\.^$()\[\]])/\\$1/g;
         $replace =~ s/%[0-9]+/(.+)/ig;
 
         my @avals;
@@ -812,7 +817,7 @@ sub _getAttrKeys
             # check the data corresponds to the object's id
             for (my $i = 0; $i < @avals; $i++) {
                 if ($dvals =~ /^$avals[$i]$/i) {
-                    push(@attrkeys, $data[$obj->{attr}{$attr}->{id}[0]->{column}[0]]);
+                    push(@attrkeys, $data[$oconf->{attr}{$attr}->{id}[0]->{column}[0]]);
                     splice(@avals, $i, 1);
                     last;
 		}
