@@ -87,6 +87,44 @@ sub rollback
     return $rc;
 }
 
+=pod
+
+=head2 hashPasswd($passwd, $salt)
+
+add hash schema at the head of hashed password.
+
+=cut
+
+sub hashPasswd
+{
+    my $self = shift;
+    my ($passwd, $salt) =@_;
+    my $conf = $self->{_config};
+    my $hashpw;
+
+    my ($htype, $otype) = split(/:/, $conf->{hash});
+
+    my $hashpw = $self->SUPER::hashPasswd($passwd, $salt);
+
+    if ($htype =~ /^MYSQL$/i) {
+        # get MySQL password
+        if ($self->_getConnect()) {
+            return $passwd;
+        }
+
+        my $sth = $self->{db}->prepare("select password(\'$passwd\');");
+        if (!$sth->execute) {
+            $self->log(level => 'error', message => "MySQL PASSWORD(\'$passwd\') failed: ".$sth->errstr);
+            return $passwd;
+        }
+        my @data = $sth->fetchrow_array;
+        $hashpw = $data[0];
+        $sth->finish;
+    }
+
+    return $hashpw;
+}
+
 sub _getConnect
 {
     my $self = shift;
@@ -134,39 +172,40 @@ sub _objSearch
     my $self = shift;
     my ($obj, $pkey, $suffix, $sizeLim, $filter) = @_;
     my $conf = $self->{_config};
+    my $oconf = $obj->{conf};
     my @match_entries = ();
     my @match_keys = ();
     my $rc = LDAP_SUCCESS;
 
     DO: {
         # get data of the entries
-        my $sql = "select distinct $obj->{table}[0].$obj->{id}[0]->{column}[0], $obj->{table}[0].$obj->{attr}{$obj->{rdn}[0]}->{column}[0]";
-        foreach my $attr ('objectclass', keys %{$obj->{attr}}) {
-            if (!defined($obj->{attr}{$attr}) || !defined($obj->{attr}{$attr}->{column})) {
+        my $sql = "select distinct $oconf->{table}[0].$oconf->{id}[0]->{column}[0], $oconf->{table}[0].$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]";
+        foreach my $attr ('objectclass', keys %{$oconf->{attr}}) {
+            if (!defined($oconf->{attr}{$attr}) || !defined($oconf->{attr}{$attr}->{column})) {
                 next;
             }
-            $sql = "$sql, $obj->{table}[0].$obj->{attr}{$attr}->{column}[0]";
+            $sql = "$sql, $oconf->{table}[0].$oconf->{attr}{$attr}->{column}[0]";
         }
 
         # exchange the LDAP filter to SQL
         my $from;
         my $where;
-        if (!$self->_filter2sql($obj, $filter, \$from, \$where)) {
+        if (!$self->_filter2sql($oconf, $filter, \$from, \$where)) {
             $rc = LDAP_FILTER_ERROR;
             @match_entries = ();
             last DO;
         }
 
         # entries below suffix
-        if ($pkey && defined($obj->{container}) && !defined($obj->{container}[0]->{rdn})) {
-            my $pobj = $conf->{object}{$obj->{container}[0]->{oname}[0]};
-            if (defined($obj->{container}[0]->{fromtbls})) {
-                $from = "$from,$obj->{container}[0]->{fromtbls}[0]";
+        if ($pkey && defined($oconf->{container}) && !defined($oconf->{container}[0]->{rdn})) {
+            my $pobj = $conf->{object}{$oconf->{container}[0]->{oname}[0]};
+            if (defined($oconf->{container}[0]->{fromtbls})) {
+                $from = "$from,$oconf->{container}[0]->{fromtbls}[0]";
             }
 
             $where = "$where and $pobj->{table}[0].$pobj->{id}[0]->{column}[0] = \'$pkey\'";
-            if (defined($obj->{container}[0]->{joinwhere})) {
-                $where = "$where and $obj->{container}[0]->{joinwhere}[0]";
+            if (defined($oconf->{container}[0]->{joinwhere})) {
+                $where = "$where and $oconf->{container}[0]->{joinwhere}[0]";
             }
         }
 
@@ -198,17 +237,17 @@ sub _objSearch
                 next;
             }
 
-            $entry = "dn: $obj->{rdn}[0]=".shift(@data).",$suffix\n";
+            $entry = "dn: $oconf->{rdn}[0]=".shift(@data).",$suffix\n";
 
             my $j = 0;
-            foreach my $attr ('objectclass', keys %{$obj->{attr}}) {
+            foreach my $attr ('objectclass', keys %{$oconf->{attr}}) {
                 if ($attr =~ /^objectclass$/i) {
-                    foreach my $oc (@{$obj->{oc}}) {
+                    foreach my $oc (@{$oconf->{oc}}) {
                         $entry = $entry."objectclass: $oc\n";
                     }
-                } elsif (!defined($obj->{attr}->{$attr})) {
+                } elsif (!defined($oconf->{attr}->{$attr})) {
                     next;
-                } elsif (defined($obj->{attr}{$attr}->{column})) {
+                } elsif (defined($oconf->{attr}{$attr}->{column})) {
                     if ($data[$j]) {
                         my $value = $data[$j];
 
@@ -217,7 +256,7 @@ sub _objSearch
                     $j++;
                 } else {
                     # get the values from the attribute's table
-                    my $values = $self->_getAttrValues($obj, $attr, $key);
+                    my $values = $self->_getAttrValues($oconf, $attr, $key);
                     if (!defined($values)) {
                         $rc = LDAP_OPERATIONS_ERROR;
                         @match_entries = ();
@@ -260,6 +299,7 @@ sub _objModify
     my $self = shift;
     my ($obj, $pkey, $dn, @list) = @_;
     my $conf = $self->{_config};
+    my $oconf = $obj->{conf};
     my $rc = LDAP_SUCCESS;
 
     DO: {
@@ -295,12 +335,12 @@ sub _objModify
                 push(@values, shift @list);
             }
 
-            if (!defined($obj->{attr}{$attr})) {
+            if (!defined($oconf->{attr}{$attr})) {
                 next;
             }
 
             # can't modify the attribute for rdn
-            if ($attr eq $obj->{rdn}[0]) {
+            if ($attr eq $oconf->{rdn}[0]) {
                 if ($action ne "REPLACE" || @values != 1 || !($entry =~ /^$attr: $values[0]$/mi)) {
                     $rc = LDAP_CONSTRAINT_VIOLATION;
                     last DO;
@@ -323,13 +363,13 @@ sub _objModify
                     last DO;
                 }
 
-                if (defined($obj->{attr}{$attr}->{column})) {
+                if (defined($oconf->{attr}{$attr}->{column})) {
                     # the attribute must not exist
                     if ($entry =~ /^$attr: /mi) {
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                         last DO;
                     }
-                    $sql = "update $obj->{table}[0] set $obj->{attr}{$attr}->{column}[0] = \'$values[0]\' where $obj->{id}[0]->{column}[0] = \'$key\'";
+                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$values[0]\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
 
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], decode('utf8', $sql));
@@ -343,9 +383,9 @@ sub _objModify
                     $sth->finish;
                 }
 
-                if (defined($obj->{attr}{$attr}->{addproc})) {
+                if (defined($oconf->{attr}{$attr}->{addproc})) {
                     foreach $value (@values) {
-                        if ($self->_addAttrValues($obj, $key, $attr, $value)) {
+                        if ($self->_addAttrValues($oconf, $key, $attr, $value)) {
                             $rc = LDAP_OPERATIONS_ERROR;
                             last DO;
                         }
@@ -360,12 +400,12 @@ sub _objModify
                     }
                 }
 
-                if (defined($obj->{attr}{$attr}->{column})) {
+                if (defined($oconf->{attr}{$attr}->{column})) {
                     if (@values && (@values > 1 || $values[0] && !($entry =~ /^$attr: $values[0]$/mi))) {
                         $rc = LDAP_NO_SUCH_ATTRIBUTE;
                         last DO;
                     }
-                    $sql = "update $obj->{table}[0] set $obj->{attr}{$attr}->{column}[0] = \'\' where $obj->{id}[0]->{column}[0] = \'$key\'";
+                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
 
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], decode('utf8', $sql));
@@ -379,26 +419,26 @@ sub _objModify
                     $sth->finish;
                 }
 
-                if (defined($obj->{attr}{$attr}->{delproc})) {
+                if (defined($oconf->{attr}{$attr}->{delproc})) {
                     if (!@values || !$values[0]) {
                         @values = ($entry =~ /^$attr: (.*)$/mgi);
                     }
 
                     foreach $value (@values) {
-                        if ($self->_delAttrValues($obj, $key, $attr, $value)) {
+                        if ($self->_delAttrValues($oconf, $key, $attr, $value)) {
                             $rc = LDAP_OPERATIONS_ERROR;
                             last DO;
                         }
                     }
                 }
             } elsif( $action eq "REPLACE" ) {
-                if (defined($obj->{attr}{$attr}->{column})) {
+                if (defined($oconf->{attr}{$attr}->{column})) {
                     # the attribute must not have more than two value
                     if (@values > 1) {
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                         last DO;
                     }
-                    $sql = "update $obj->{table}[0] set $obj->{attr}{$attr}->{column}[0] = \'$values[0]\' where $obj->{id}[0]->{column}[0] = \'$key\'";
+                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$values[0]\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
 
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], decode('utf8', $sql));
@@ -412,7 +452,7 @@ sub _objModify
                     $sth->finish;
                 }
 
-                if (defined($obj->{attr}{$attr}->{addproc}) || defined($obj->{attr}{$attr}->{delproc})) {
+                if (defined($oconf->{attr}{$attr}->{addproc}) || defined($oconf->{attr}{$attr}->{delproc})) {
                     my @old_vals = ($entry =~ /^$attr: (.*)$/mgi);
                     my @existing_vals;
 
@@ -429,7 +469,7 @@ sub _objModify
                             push(@existing_vals, $old_val);
                             next;
                         }
-                        if ($self->_delAttrValues($obj, $key, $attr, $old_val)) {
+                        if ($self->_delAttrValues($oconf, $key, $attr, $old_val)) {
                             $rc = LDAP_OPERATIONS_ERROR;
                             last DO;
                         }
@@ -440,7 +480,7 @@ sub _objModify
                         if (grep(/^$value$/i, @existing_vals)) {
                             next;
                         }
-                        if ($self->_addAttrValues($obj, $key, $attr, $value)) {
+                        if ($self->_addAttrValues($oconf, $key, $attr, $value)) {
                             $rc = LDAP_OPERATIONS_ERROR;
                             last DO;
                         }
@@ -470,6 +510,7 @@ sub _objAdd
     my $self = shift;
     my ($obj, $pkey, $dn, $entryStr) = @_;
     my $conf = $self->{_config};
+    my $oconf = $obj->{conf};
     my $rc = LDAP_SUCCESS;
 
     DO: {
@@ -496,15 +537,15 @@ sub _objAdd
         my @values;
         my %attrs;
 
-        if (defined($obj->{id}[0]->{sequence})) {
-            push(@cols, $obj->{id}[0]->{column}[0]);
-            push(@values, $obj->{id}[0]->{sequence}[0]);
+        if (defined($oconf->{id}[0]->{sequence})) {
+            push(@cols, $oconf->{id}[0]->{column}[0]);
+            push(@values, $oconf->{id}[0]->{sequence}[0]);
         }
 
         foreach (split(/\n/, $entryStr)) {
             my ($attr, $value) = split(/: /);
             $attr = lc($attr);
-            if (!defined($obj->{attr}{$attr})) {
+            if (!defined($oconf->{attr}{$attr})) {
                 next;
             }
 
@@ -512,20 +553,20 @@ sub _objAdd
             $value =~ s/'/''/g;
             $value =~ s/\\/\\\\/g;
 
-            if (defined($obj->{attr}{$attr}->{column})) {
-                if (!grep(/^$obj->{attr}{$attr}->{column}[0]$/, @cols)) {
-                    push(@cols, $obj->{attr}{$attr}->{column}[0]);
+            if (defined($oconf->{attr}{$attr}->{column})) {
+                if (!grep(/^$oconf->{attr}{$attr}->{column}[0]$/, @cols)) {
+                    push(@cols, $oconf->{attr}{$attr}->{column}[0]);
                     push(@values, "\'$value\'");
                 }
             }
-            if (defined($obj->{attr}{$attr}->{addproc})) {
+            if (defined($oconf->{attr}{$attr}->{addproc})) {
                 push(@{$attrs{$attr}}, $value);
             }
         }
 
         # get the storage-specific information
         my @si_values = (); 
-        foreach my $strginfo (@{$obj->{strginfo}}) {
+        foreach my $strginfo (@{$oconf->{strginfo}}) {
             my $value = $self->_getStrgInfoValue($strginfo, $dn, $entryStr);
             push(@si_values, $value);
 
@@ -537,7 +578,7 @@ sub _objAdd
             }
         }
 
-        my $sql = "insert into $obj->{table}[0](".join(', ', @cols).") values(".join(', ', @values).")";
+        my $sql = "insert into $oconf->{table}[0](".join(', ', @cols).") values(".join(', ', @values).")";
 
         # multibyte value
         $sql = encode($conf->{mbcode}[0], decode('utf8', $sql));
@@ -571,7 +612,7 @@ sub _objAdd
         # add the values in the attribute's table
         foreach my $attr (keys %attrs) {
             foreach my $value (@{$attrs{$attr}}) {
-                if ($rc = $self->_addAttrValues($obj, $key, $attr, $value)) {
+                if ($rc = $self->_addAttrValues($oconf, $key, $attr, $value)) {
                     $self->log(level => 'error', message => "Adding values of $attr failed");
                     $rc = LDAP_OPERATIONS_ERROR;
                     last DO;
@@ -580,12 +621,12 @@ sub _objAdd
         }
 
         # add the storage-specific information
-        for (my $i = 0; $i < @{$obj->{strginfo}}; $i++) {
-            if (!defined(${$obj->{strginfo}}[$i]->{addproc})) {
+        for (my $i = 0; $i < @{$oconf->{strginfo}}; $i++) {
+            if (!defined(${$oconf->{strginfo}}[$i]->{addproc})) {
                 next;
             }
 
-            foreach my $addproc (@{${$obj->{strginfo}}[$i]->{addproc}}) {
+            foreach my $addproc (@{${$oconf->{strginfo}}[$i]->{addproc}}) {
 	        $sql = $addproc;
                 $sql =~ s/\%o/$key/g;
                 $sql =~ s/\%v/$si_values[$i]/g;
@@ -604,8 +645,8 @@ sub _objAdd
         }
 
         # add the link with container
-        if (defined($obj->{container}) && defined($obj->{container}[0]->{addproc})) {
-            $sql = $obj->{container}[0]->{addproc}[0];
+        if (defined($oconf->{container}) && defined($oconf->{container}[0]->{addproc})) {
+            $sql = $oconf->{container}[0]->{addproc}[0];
             $sql =~ s/\%o/$key/g;
             $sql =~ s/\%c/$pkey/g;
             $sth = $self->{db}->prepare("$sql");
@@ -638,6 +679,7 @@ sub _objDelete
     my $self = shift;
     my ($obj, $pkey, $dn) = @_;
     my $conf = $self->{_config};
+    my $oconf = $obj->{conf};
     my $rc = LDAP_SUCCESS;
 
     DO: {
@@ -664,7 +706,7 @@ sub _objDelete
         my $sth;
 
         # delete the storage-specific information
-        foreach my $strginfo (@{$obj->{strginfo}}) {
+        foreach my $strginfo (@{$oconf->{strginfo}}) {
             if (!defined($strginfo->{delproc})) {
                 next;
             }
@@ -688,9 +730,9 @@ sub _objDelete
         }
 
         # delete the values from the attribute's table
-        foreach my $attr (keys %{$obj->{attr}}) {
-            if (defined($obj->{attr}{$attr}) && defined($obj->{attr}{$attr}->{delproc})) {
-                if ($rc = $self->_delAttrValues($obj, $key, $attr, '')) {
+        foreach my $attr (keys %{$oconf->{attr}}) {
+            if (defined($oconf->{attr}{$attr}) && defined($oconf->{attr}{$attr}->{delproc})) {
+                if ($rc = $self->_delAttrValues($oconf, $key, $attr, '')) {
                     $rc = LDAP_OPERATIONS_ERROR;
                     last DO;
                 }
@@ -698,8 +740,8 @@ sub _objDelete
         }
 
         # delete the link with container
-        if (defined($obj->{container}) && defined($obj->{container}[0]->{delproc})) {
-            $sql = $obj->{container}[0]->{delproc}[0];
+        if (defined($oconf->{container}) && defined($oconf->{container}[0]->{delproc})) {
+            $sql = $oconf->{container}[0]->{delproc}[0];
             $sql =~ s/\%o/$key/g;
             $sql =~ s/\%c/$pkey/g;
             $sth = $self->{db}->prepare("$sql");
@@ -712,7 +754,7 @@ sub _objDelete
         }
 
         # delete the appropriate record from the object's table
-        $sql = "delete from $obj->{table}[0] where $obj->{id}[0]->{column}[0] = \'$key\'";
+        $sql = "delete from $oconf->{table}[0] where $oconf->{id}[0]->{column}[0] = \'$key\'";
         $sth = $self->{db}->prepare($sql);
         if (!$sth->execute) {
             $self->log(level => 'error', message => "Deleting entry by \"$sql\" failed: ".$sth->errstr);
@@ -734,31 +776,38 @@ sub _getParentRdn
     my $self = shift;
     my ($obj, $key) = @_;
     my $conf = $self->{_config};
+    my $oconf = $obj->{conf};
     my $selexpr;
     my $from;
     my $where;
 
-    if (defined($obj->{container}[0]->{rdn})) {
-        return $obj->{container}[0]->{rdn}[0];
+    if (defined($oconf->{container}[0]->{rdn})) {
+        return $oconf->{container}[0]->{rdn}[0];
     }
-    if (!defined($obj->{container}[0]->{oname})) {
+    if (!defined($oconf->{container}[0]->{oname})) {
         return undef;
     }
 
-    my $parent = $conf->{object}{$obj->{container}[0]->{oname}[0]};
-    $selexpr = "$parent->{table}[0].$parent->{id}[0]->{column}[0],$parent->{table}[0].$parent->{attr}{$parent->{rdn}[0]}->{column}[0]";
+    my $poconf = $self->{object}{$obj->{parent}}->{conf};
 
-    $from = $obj->{table}[0];
-    if (defined($obj->{container}[0]->{fromtbls})) {
-        $from = "$from,$obj->{container}[0]->{fromtbls}[0]";
+    if (defined($poconf->{entry})) {
+        return $poconf->{entry}[0]->{rdn}[0];
     }
 
-    $where = "$obj->{table}[0].$obj->{id}[0]->{column}[0] = \'$key\'";
-    if (defined($obj->{container}[0]->{joinwhere})) {
-        $where = "$where and $obj->{container}[0]->{joinwhere}[0]";
+    $selexpr = "$poconf->{table}[0].$poconf->{id}[0]->{column}[0],$poconf->{table}[0].$poconf->{attr}{$poconf->{rdn}[0]}->{column}[0]";
+
+    $from = $oconf->{table}[0];
+    if (defined($oconf->{container}[0]->{fromtbls})) {
+        $from = "$from,$oconf->{container}[0]->{fromtbls}[0]";
+    }
+
+    $where = "$oconf->{table}[0].$oconf->{id}[0]->{column}[0] = \'$key\'";
+    if (defined($oconf->{container}[0]->{joinwhere})) {
+        $where = "$where and $oconf->{container}[0]->{joinwhere}[0]";
     }
 
     my $sql = "select $selexpr from $from where $where";
+print STDERR "sql: $sql\n";
     my $sth = $self->{db}->prepare($sql);
     if (!$sth->execute) {
         $self->log(level => 'error', message => "Getting rdn by \"$sql\" failed: ".$sth->errstr);
@@ -769,35 +818,37 @@ sub _getParentRdn
     my @data = $sth->fetchrow_array;
     $sth->finish;
 
-    return ("$parent->{rdn}[0]=$data[1]", $data[0]);
+    return ("$poconf->{rdn}[0]=$data[1]", $data[0]);
 }
 
 sub _getAttrValues
 {
     my $self = shift;
-    my ($obj, $attr, $key) = @_;
+    my ($oconf, $attr, $key) = @_;
     my $conf = $self->{_config};
-    my $attrobj;
+    my $aobj = undef;
+    my $aoconf = undef;
     my $attrStr = '';
     my $selexpr;
     my $from;
     my $where;
 
-    if (defined($obj->{attr}{$attr}->{oname})) {
-        $attrobj = $conf->{object}->{$obj->{attr}{$attr}->{oname}[0]};
-        $selexpr = "$attrobj->{table}[0].$attrobj->{id}[0]->{column}[0], $attrobj->{table}[0].$attrobj->{attr}{$attrobj->{rdn}[0]}->{column}[0]";
+    if (defined($oconf->{attr}{$attr}->{oname})) {
+        $aobj = $self->{object}{$oconf->{attr}{$attr}->{oname}[0]};
+        $aoconf = $aobj->{conf};
+        $selexpr = "$aoconf->{table}[0].$aoconf->{id}[0]->{column}[0], $aoconf->{table}[0].$aoconf->{attr}{$aoconf->{rdn}[0]}->{column}[0]";
     } else {
-        $selexpr = $obj->{attr}{$attr}->{selexpr}[0];
+        $selexpr = $oconf->{attr}{$attr}->{selexpr}[0];
     }
 
-    $from = $obj->{table}[0];
-    if (defined($obj->{attr}{$attr}->{fromtbls})) {
-        $from = "$from,$obj->{attr}{$attr}->{fromtbls}[0]";
+    $from = $oconf->{table}[0];
+    if (defined($oconf->{attr}{$attr}->{fromtbls})) {
+        $from = "$from,$oconf->{attr}{$attr}->{fromtbls}[0]";
     }
 
-    $where = "$obj->{table}[0].$obj->{id}[0]->{column}[0] = \'$key\'";
-    if (defined($obj->{attr}{$attr}->{joinwhere})) {
-        $where = "$where and $obj->{attr}{$attr}->{joinwhere}[0]";
+    $where = "$oconf->{table}[0].$oconf->{id}[0]->{column}[0] = \'$key\'";
+    if (defined($oconf->{attr}{$attr}->{joinwhere})) {
+        $where = "$where and $oconf->{attr}{$attr}->{joinwhere}[0]";
     }
 
     my $sql = "select $selexpr from $from where $where";
@@ -811,8 +862,8 @@ sub _getAttrValues
     while (my @data = $sth->fetchrow_array) {
         if (!@data) {
             next;
-        } elsif ($attrobj) {
-            $attrStr = $attrStr."$attr: $attrobj->{rdn}[0]=$data[1],".$self->_getParentDn($attrobj, $data[0])."\n";
+        } elsif ($aoconf) {
+            $attrStr = $attrStr."$attr: $aoconf->{rdn}[0]=$data[1],".$self->_getParentDn($aobj, $data[0])."\n";
         } else {
             $attrStr = $attrStr."$attr: $data[0]\n";
         }
@@ -825,25 +876,25 @@ sub _getAttrValues
 sub _addAttrValues
 {
     my $self = shift;
-    my ($obj, $key, $attr, $value) = @_;
+    my ($oconf, $key, $attr, $value) = @_;
     my $conf = $self->{_config};
 
-    if (!defined($obj->{attr}{$attr}->{addproc})) {
+    if (!defined($oconf->{attr}{$attr}->{addproc})) {
         return -1;
     }
 
-    foreach my $addproc (@{$obj->{attr}{$attr}->{addproc}}) {
+    foreach my $addproc (@{$oconf->{attr}{$attr}->{addproc}}) {
         my $sql = $addproc;
         $sql =~ s/\%o/$key/g;
-        if (defined($obj->{attr}{$attr}->{oname})) {
-            my ($rc, $attrobj, $attrpkey) = $self->_getObject($value);
+        if (defined($oconf->{attr}{$attr}->{oname})) {
+            my ($rc, $aobj, $attrpkey) = $self->_getObject($value);
             if ($rc) {
                 return -1;
             }
 
             my $attrkey;
 
-            ($rc, $attrkey) =$self->_baseSearch($attrobj, $attrpkey, $value, 0, 0, 1, 0, undef);
+            ($rc, $attrkey) =$self->_baseSearch($aobj, $attrpkey, $value, 0, 0, 1, 0, undef);
             if ($rc) {
                 $self->log(level => 'error', message => "Can't get id of $value in the table");
                 return -1;
@@ -874,26 +925,26 @@ sub _addAttrValues
 sub _delAttrValues
 {
     my $self = shift;
-    my ($obj, $key, $attr, $value) = @_;
+    my ($oconf, $key, $attr, $value) = @_;
     my $conf = $self->{_config};
 
-    if (!defined($obj->{attr}{$attr}->{delproc})) {
+    if (!defined($oconf->{attr}{$attr}->{delproc})) {
         return 1;
     }
 
-    foreach my $delproc (@{$obj->{attr}{$attr}->{delproc}}) {
+    foreach my $delproc (@{$oconf->{attr}{$attr}->{delproc}}) {
         my $sql = $delproc;
         $sql =~ s/\%o/$key/g;
         if ($value) {
-            if (defined($obj->{attr}{$attr}->{oname})) {
-                my ($rc, $attrobj, $attrpkey) = $self->_getObject($value);
+            if (defined($oconf->{attr}{$attr}->{oname})) {
+                my ($rc, $aobj, $attrpkey) = $self->_getObject($value);
                 if ($rc) {
                     return -1;
                 }
 
                 my $attrkey;
 
-                ($rc, $attrkey) =$self->_baseSearch($attrobj, $attrpkey, $value, 0, 0, 1, 0, undef);
+                ($rc, $attrkey) =$self->_baseSearch($aobj, $attrpkey, $value, 0, 0, 1, 0, undef);
                 if ($rc) {
 		    $self->log(level => 'error', message => "Can't get id of $value from the table");
                     return -1;
@@ -927,12 +978,12 @@ sub _delAttrValues
 sub _filter2sql
 {
     my $self = shift;
-    my ($obj, $filter, $from, $where) = @_;
+    my ($oconf, $filter, $from, $where) = @_;
     my ($type, $attr, $value);
     my $conf = $self->{_config};
 
     if (!$filter) {
-        ${$from} = $obj->{table}[0];
+        ${$from} = $oconf->{table}[0];
         ${$where} = 1;
         return 1;
     }
@@ -942,8 +993,8 @@ sub _filter2sql
 
     if ($op eq 'and' || $op eq 'or') {
         my ($lfrom, $lwhere, $rfrom, $rwhere);
-        my $left = $self->_filter2sql($obj, @{$args}[0], \$lfrom, \$lwhere);
-        my $right = $self->_filter2sql($obj, @{$args}[1], \$rfrom, \$rwhere);
+        my $left = $self->_filter2sql($oconf, @{$args}[0], \$lfrom, \$lwhere);
+        my $right = $self->_filter2sql($oconf, @{$args}[1], \$rfrom, \$rwhere);
         if (!$left || !$right) {
             return 0;
         }
@@ -958,7 +1009,7 @@ sub _filter2sql
         }
         return 1;
     } elsif ($op eq 'not'){
-        if (!$self->_filter2sql($obj, $args, $from, $where)) {
+        if (!$self->_filter2sql($oconf, $args, $from, $where)) {
             return 0;
         }
 
@@ -985,7 +1036,7 @@ sub _filter2sql
             $value = '%';
         }
 
-        if (defined($obj->{attr}{$attr}) && defined($obj->{attr}{$attr}->{oname})) {
+        if (defined($oconf->{attr}{$attr}) && defined($oconf->{attr}{$attr}->{oname})) {
             return 0;
         }
     }
@@ -994,38 +1045,38 @@ sub _filter2sql
     $value =~ s/'/''/g;
     $value =~ s/\\/\\\\/g;
 
-    ${$from} = $obj->{table}[0];
+    ${$from} = $oconf->{table}[0];
     if ($attr =~ /^objectclass$/i) {
         # check object class in filter
         $value =~ s/%/.*/;
-        if (grep(/^$value$/i, @{$obj->{oc}})) {
+        if (grep(/^$value$/i, @{$oconf->{oc}})) {
             ${$where} = '1 = 1';
         } else {
             ${$where} = '1 = 0';
         }
-    } elsif (!defined($obj->{attr}{$attr})) {
+    } elsif (!defined($oconf->{attr}{$attr})) {
         # attribute doesn't exit
         ${$where} = '1 = 0';
-    } elsif (defined($obj->{attr}{$attr}->{column})) {
-        ${$where} = "$obj->{table}[0].$obj->{attr}{$attr}->{column}[0] $type \'$value\'";
-    } elsif (defined($obj->{attr}{$attr}->{oname})) {
-        my $attrobj = $conf->{object}{$obj->{attr}{$attr}->{oname}[0]};
-        if (defined($obj->{attr}{$attr}->{fromtbls})) {
-            ${$from} = "${$from},$obj->{attr}{$attr}->{fromtbls}[0]";
+    } elsif (defined($oconf->{attr}{$attr}->{column})) {
+        ${$where} = "$oconf->{table}[0].$oconf->{attr}{$attr}->{column}[0] $type \'$value\'";
+    } elsif (defined($oconf->{attr}{$attr}->{oname})) {
+        my $attrobj = $conf->{object}{$oconf->{attr}{$attr}->{oname}[0]};
+        if (defined($oconf->{attr}{$attr}->{fromtbls})) {
+            ${$from} = "${$from},$oconf->{attr}{$attr}->{fromtbls}[0]";
         }
 
         ($value) = ($value =~ /^$attrobj->{rdn}[0]=([^,]*)/i);
         ${$where} = "$attrobj->{table}[0].$attrobj->{attr}{$attrobj->{rdn}[0]}->{column}[0] $type \'$value\'";
-        if (defined($obj->{attr}{$attr}->{joinwhere})) {
-            ${$where} = "${$where} and $obj->{attr}{$attr}->{joinwhere}[0]";
+        if (defined($oconf->{attr}{$attr}->{joinwhere})) {
+            ${$where} = "${$where} and $oconf->{attr}{$attr}->{joinwhere}[0]";
         }
     } else {
-        if (defined($obj->{attr}{$attr}->{fromtbls})) {
-            ${$from} = "${$from},$obj->{attr}{$attr}->{fromtbls}[0]";
+        if (defined($oconf->{attr}{$attr}->{fromtbls})) {
+            ${$from} = "${$from},$oconf->{attr}{$attr}->{fromtbls}[0]";
         }
-        ${$where} = "$obj->{attr}{$attr}->{selexpr} $type \'$value\'";
-        if (defined($obj->{attr}{$attr}->{joinwhere})) {
-            ${$where} = "${$where} and $obj->{attr}{$attr}->{joinwhere}[0]";
+        ${$where} = "$oconf->{attr}{$attr}->{selexpr} $type \'$value\'";
+        if (defined($oconf->{attr}{$attr}->{joinwhere})) {
+            ${$where} = "${$where} and $oconf->{attr}{$attr}->{joinwhere}[0]";
         }
     }
 
