@@ -146,7 +146,7 @@ sub start_server {
   unlink("$TEMPDIR/syncfail-SQL.log");
   unlink("$TEMPDIR/syncfail-CSV.log");
 
-  foreach my $file ('user.csv', 'group.csv', 'division.csv', 'department.csv', 'category.csv') {
+  foreach my $file ('user.csv', 'group.csv', 'division.csv', 'department.csv', 'category.csv', 'company.csv') {
     open CSV, "> $TESTCSV/$file";
     close CSV;
   }
